###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__ = 'Donovan Parks'
__copyright__ = 'Copyright 2014'
__credits__ = ['Donovan Parks']
__license__ = 'GPL3'
__maintainer__ = 'Donovan Parks'
__email__ = 'donovan.parks@gmail.com'

import os
import tempfile
import logging

from biolib.external.execute import check_on_path


class Diamond(object):
    """Wrapper for running diamond."""

    def __init__(self, cpus=1):
        """Initialization.

        Parameters
        ----------
        cpus : int
            Number of cpus to use.
        """
        self.logger = logging.getLogger('timestamp')

        check_on_path('diamond')

        self.cpus = cpus
        
        self.output_fmt = { 'diamond_daa': '100',
                            'standard': '6',
                            'custom': '6 qseqid qlen sseqid stitle slen length pident evalue bitscore'}

    def make_database(self, prot_file, db_file):
        """Make diamond database.

        Parameters
        ----------
        prot_file : str
            Fasta file with protein sequences.
        db_file : str
            Desired name of Diamond database.
        """

        cmd = 'diamond makedb --quiet -p %d --in %s -d %s' % (self.cpus, prot_file, db_file)
        os.system(cmd)

    def blastp(self, prot_file, db_file, evalue, per_identity, per_aln_len, max_target_seqs, output_file, output_fmt='standard', tmp_dir=None, chunk_size=None, block_size=None):
        """Apply diamond blastp to a set of protein sequences.

        Parameters
        ----------
        prot_file : str
            Fasta file with protein sequences.
        db_file : str
            Diamond database of protein sequences.
        evalue : float
            E-value threshold used by blast.
        per_identity : float
            Percent identity threshold used by blast [0, 100].
        per_aln_len : float
            Percent query coverage threshold for reporting hits [0, 100].
        max_target_seqs : int
            Maximum number of hits to report per sequence.
        output_file : str
            Desired name of output file.
        output_fmt : str
            Desired output format (diamond_daa, standard, custom)
        tmp_dir : str
            Directory to store temporary files.
        chunk_size : int
            Number of chunks for index processing.
        block_size : int
            Sequence block size in billions of letters.
        """
        
        assert(output_fmt in self.output_fmt)

        if db_file.endswith('.dmnd'):
            db_file = db_file[0:db_file.rfind('.dmnd')]

        args = ''
        if tmp_dir:
             args += '-t %s' % tmp_dir

        if chunk_size:
             args += ' -c %d' % chunk_size
             
        if block_size:
            args += ' -b %d' % block_size

        cmd = "diamond blastp --quiet --seg no -p %d -q %s -d %s -e %g --id %f --query-cover %f -k %d -o %s -f %s %s" % (self.cpus,
                                                                                                                    prot_file,
                                                                                                                    db_file,
                                                                                                                    evalue,
                                                                                                                    per_identity,
                                                                                                                    per_aln_len,
                                                                                                                    max_target_seqs,
                                                                                                                    output_file,
                                                                                                                    self.output_fmt[output_fmt],
                                                                                                                    args)

        os.system(cmd)

    def blastx(self, nt_file, db_file, evalue, per_identity, max_target_seqs, diamond_daa_file):
        """Apply diamond blastx to a set of nucleotide sequences.

        Parameters
        ----------
        nt_file : str
            Fasta file with nucleotide sequences.
        db_file : str
            Diamond database of protein sequences.
        evalue : float
            E-value threshold used by blast.
        per_identity : float
            Percent identity threshold used by blast [0, 100].
        max_target_seqs : int
            Maximum number of hits to report per sequence.
        diamond_daa_file : str
            Desired name of Diamond data file.
        """

        if db_file.endswith('.dmnd'):
            db_file = db_file[0:db_file.rfind('.dmnd')]

        cmd = 'diamond blastx --quiet --seg no -p %d -t %s -q %s -d %s -e %f --id %f -k %d -a %s' % (self.cpus,
                                                                                                        tempfile.gettempdir(),
                                                                                                        nt_file,
                                                                                                        db_file,
                                                                                                        evalue,
                                                                                                        per_identity,
                                                                                                        max_target_seqs,
                                                                                                        diamond_daa_file)

        os.system(cmd)

    def view(self, diamond_daa_file, output_table, compress=False):
        """Generate flat file from diamond DAA file.

        Parameters
        ----------
        diamond_daa_file : str
            Diamond DAA file.
        output_table : str
            Diamond database of protein sequeces.
        compress : boolean
            Flag indicating if output table should be compressed.
        """

        cmd = 'diamond view --quiet -p %d -a %s -o %s --compress %d' % (self.cpus,
                                                                            diamond_daa_file,
                                                                            output_table,
                                                                            compress)
        os.system(cmd)



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__ = 'Donovan Parks'
__copyright__ = 'Copyright 2014'
__credits__ = ['Donovan Parks']
__license__ = 'GPL3'
__maintainer__ = 'Donovan Parks'
__email__ = 'donovan.parks@gmail.com'

import os
import errno
import sys
import logging
import ntpath
import re
import gzip

from numpy import (abs as np_abs,
                   array as np_array)


def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    http://stackoverflow.com/questions/3041986/python-command-line-yes-no-input

    Parameters
    ----------
    question : str
        Prompt presented to the user.
    default : str
        Presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    Returns
    -------
    boolean
        True for "yes", False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")


def is_float(s):
    """Check if a string can be converted to a float.

    Parameters
    ----------
    s : str
        String to evaluate.

    Returns
    -------
    boolean
        True if string can be converted, else False.
    """

    try:
        float(s)
    except ValueError:
        return False

    return True


def find_nearest(array, value):
    """Find nearest array element to a given value.

    Parameters
    ----------
    array : iterable
        List of values.
    value : float
        Target value

    Returns
    -------
    array element
        Closest element in 'array' to 'value'.

    """
    idx = (np_abs(np_array(array) - value)).argmin()
    return array[idx]


def concatenate_files(input_files, output_file, common_header=False):
    """Concatenate several files into a single file.

    Creates a compressed file if the extension of
    the output file ends with .gz.

    Parameters
    ----------
    input_files : iterable
        Files to concatenate.
    output_file : str
        Name of output file.
    common_header : boolean
        If True, only write first line of first file.
    """

    if output_file.endswith('.gz'):
        open_file = gzip.open
    else:
        open_file = open

    with open_file(output_file, "wb") as outfile:
        for file_no, f in enumerate(input_files):
            with open(f, "rb") as infile:
                for line_no, line in enumerate(infile):
                    if common_header and line_no == 0:
                        if file_no == 0:
                            outfile.write(line)
                        else:
                            pass
                    else:
                        outfile.write(line)
                        
                # force newline between files
                if line[-1] != '\n':
                    outfile.write('\n')

                        
def alphanumeric_sort(l):
    """Sorts the given iterable alphanumerically.

    http://stackoverflow.com/questions/2669059/how-to-sort-alpha-numeric-set-in-python

    Parameters
    ----------
    l : iterable
        The iterable to be sorted.

    Returns
    -------
    iterable
        Iterable sorted alphanumerically.
    """
    convert = lambda text: int(text) if text.isdigit() else text
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return sorted(l, key=alphanum_key)


def check_file_exists(input_file):
    """Check if file exists."""
    if not os.path.exists(input_file) or not os.path.isfile(input_file):
        logger = logging.getLogger('timestamp')
        logger.error('Input file does not exists: ' + input_file + '\n')
        sys.exit()


def check_dir_exists(input_dir):
    """Check if directory exists."""
    if not os.path.exists(input_dir) or not os.path.isdir(input_dir):
        logger = logging.getLogger('timestamp')
        logger.error('Input directory does not exists: ' + input_dir + '\n')
        sys.exit()


def make_sure_path_exists(path):
    """Create directory if it does not exist."""

    if not path:
        # lack of a path qualifier is acceptable as this
        # simply specifies the current directory
        return

    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            logger = logging.getLogger('timestamp')
            logger.error('Specified path could not be created: ' + path + '\n')
            sys.exit()


def remove_extension(filename, extension=None):
    """Remove extension from filename.

    A specific extension can be specified, otherwise
    the extension is taken as all characters after the
    last period.
    """
    f = ntpath.basename(filename)

    if extension and f.endswith(extension):
        f = f[0:f.rfind(extension)]
    else:
        f = os.path.splitext(f)[0]

    if f[-1] == '.':
        f = f[0:-1]

    return f



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__ = 'Donovan Parks'
__copyright__ = 'Copyright 2015'
__credits__ = ['Donovan Parks']
__license__ = 'GPL3'
__maintainer__ = 'Donovan Parks'
__email__ = 'donovan.parks@gmail.com'

from abstract_plot import AbstractPlot

from matplotlib.ticker import NullFormatter
from matplotlib import collections

from numpy import (sum as np_sum)


class Scatter(AbstractPlot):
    """Scatter plot with optional histograms."""

    def __init__(self, options):
        AbstractPlot.__init__(self, options)

    def plot(self, x, y, xlabel, ylabel, show_histograms=False, num_bins=0, xlim=None, ylim=None):
        # set size of figure
        self.fig.clear()
        self.fig.set_size_inches(self.options.width, self.options.height)

        # setup histograms
        self.histogram_size = 0.5
        if show_histograms:
                histogram_sizeX = self.histogram_size / self.options.width
                histogram_sizeY = self.histogram_size / self.options.height
        else:
                histogram_sizeX = 0.0
                histogram_sizeY = 0.0

        padding = 0.1   # inches
        xoffset_figspace = (0.4 + padding)/self.options.width
        yoffset_figspace = (0.3 + padding)/self.options.height
        axes_scatter = self.fig.add_axes([xoffset_figspace, 
                                            yoffset_figspace,
                                            1.0 - xoffset_figspace - histogram_sizeX - (2*padding)/self.options.width, 
                                            1.0 - yoffset_figspace - histogram_sizeY - (2*padding)/self.options.height])

        axes_top_histogram = self.fig.add_axes([xoffset_figspace, 
                                                    1.0 - histogram_sizeY - padding/self.options.height,
                                                    1.0 - xoffset_figspace - histogram_sizeX - (2*padding)/self.options.width, 
                                                    histogram_sizeY])

        axes_right_histogram = self.fig.add_axes([1.0 - histogram_sizeX - padding/self.options.width, 
                                                    yoffset_figspace,
                                                    histogram_sizeX, 
                                                    1.0 - yoffset_figspace - histogram_sizeY - (2*padding)/self.options.height])

        # scatter plot
        hq_x = []
        hq_y = []
        hq_c = []
        mq_x = []
        mq_y = []
        mq_c = []
        p_x = []
        p_y = []
        p_c = []
        colors = [(178/255.0,223/255.0,138/255.0),
                (166/255.0,206/255.0,227/255.0),
                (127/255.0,127/255.0,127/255.0)]
                
        comp98 = 0
        for comp, cont in zip(x,y):
            if comp >= 98:
                comp98 += 1
            if comp >= 90 and cont <= 5:
                hq_x.append(comp)
                hq_y.append(cont)
                hq_c.append(colors[0])
            elif comp >= 70:
                mq_x.append(comp)
                mq_y.append(cont)
                mq_c.append(colors[1])
            else:
                p_x.append(comp)
                p_y.append(cont)
                p_c.append(colors[2])
                
        print 'comp98', comp98 * 100.0 / len(x)
        
        mq = axes_scatter.scatter(mq_x, mq_y, c=mq_c, s=30, lw=0.5)
        hq = axes_scatter.scatter(hq_x, hq_y, c=hq_c, s=30, lw=0.5)
        p = axes_scatter.scatter(p_x, p_y, c=p_c, s=30, lw=0.5)
        
        hq_count = len(hq_x)
        mq_count = len(mq_x)
        p_count = len(p_x)
        hq_perc = hq_count * 100.0 / (hq_count + mq_count + p_count)
        mq_perc = mq_count * 100.0 / (hq_count + mq_count + p_count)
        p_perc = p_count * 100.0 / (hq_count + mq_count + p_count)
        
        lgnd = axes_scatter.legend((hq, mq, p),
                                   ('High quality (%d genomes; %.1f%%)' % (hq_count, hq_perc), 
                                   'Medium quality (%d genomes; %.1f%%)' % (mq_count, mq_perc),
                                   'Partial (%d genomes; %.1f%%)' % (p_count, p_perc)),
                                   scatterpoints=1,
                                   loc='upper left',
                                   ncol=1,
                                   fontsize=self.options.tick_font_size,
                                   handletextpad=0,
                                   markerscale=2,
                                   frameon=False)

        axes_scatter.set_xlabel(xlabel)
        axes_scatter.set_ylabel(ylabel)
        
        if xlim:
            axes_scatter.set_xlim(xlim)
        if ylim:
            axes_scatter.set_ylim(ylim)
        
        # *** Prettify scatter plot
        for line in axes_scatter.yaxis.get_ticklines(): 
            line.set_color(self.axes_colour)
                
        for line in axes_scatter.xaxis.get_ticklines(): 
            line.set_color(self.axes_colour)
            
        for loc, spine in axes_scatter.spines.iteritems():
            spine.set_color(self.axes_colour)

        # plot histograms
        if not show_histograms:
            for a in axes_scatter.yaxis.majorTicks:
                    a.tick1On=True
                    a.tick2On=False
                
            for a in axes_scatter.xaxis.majorTicks:
                    a.tick1On=True
                    a.tick2On=False
                    
            for line in axes_scatter.yaxis.get_ticklines(): 
                line.set_color(self.axes_colour)
            
            for line in axes_scatter.xaxis.get_ticklines(): 
                line.set_color(self.axes_colour)

            for loc, spine in axes_scatter.spines.iteritems():
                    if loc in ['right','top']:
                            spine.set_color('none')
                    else:
                        spine.set_color(self.axes_colour)
            
        else: # show histograms 
            # get bin weights for percentage histogram plot
            x = [p_x, mq_x, hq_x]
            num_entries = sum([len(c) for c in x])
            weights = []
            for c in xrange(0, len(x)):
                w = [100.0/num_entries for _ in xrange(0, len(x[c]))]
                weights.append(w)
        
            # plot top histogram
            axes_top_histogram.xaxis.set_major_formatter(NullFormatter())
            #pdf, bins, patches = axes_top_histogram.hist([hq_x, mq_x, p_x], bins = num_bins, facecolor = 'grey')
            pdf, bins, patches = axes_top_histogram.hist(x, 
                                                            bins=num_bins, 
                                                            weights=weights,
                                                            color=colors[::-1], 
                                                            lw=0.5, 
                                                            histtype='bar', 
                                                            stacked=True)
                                                                                           
            max_y = max(pdf[len(x)-1])
            axes_top_histogram.yaxis.set_label_position('right')
            axes_top_histogram.set_xlim(axes_scatter.get_xlim())
            axes_top_histogram.set_yticks([int(max_y)])
            axes_top_histogram.set_yticklabels(['%d%%' % int(max_y)])
            axes_top_histogram.set_ylim([0, max_y*1.05])
            
            axes_top_histogram.yaxis.tick_right()
            

            # plot right histogram
            y = [p_y, mq_y, hq_y]
            axes_right_histogram.yaxis.set_major_formatter(NullFormatter())
            pdf, bins, patches = axes_right_histogram.hist(y, 
                                                            bins=num_bins, 
                                                            orientation='horizontal',
                                                            weights=weights,
                                                            color=colors[::-1], 
                                                            lw=0.5, 
                                                            histtype='bar', 
                                                            stacked=True)
                                                            
            max_x = max(pdf[len(y)-1])
            axes_right_histogram.set_ylim(axes_scatter.get_ylim())
            axes_right_histogram.set_xticks([int(max_x)])
            axes_right_histogram.set_xticklabels(['%d%%' % int(max_x)])
            axes_right_histogram.set_xlim([0, max_x*1.05])

            # *** Prettify histogram plot
            for a in axes_top_histogram.yaxis.majorTicks:
                    a.tick1On=False
                    a.tick2On=True
                
            for a in axes_top_histogram.xaxis.majorTicks:
                    a.tick1On=True
                    a.tick2On=False
                    
            for line in axes_top_histogram.yaxis.get_ticklines(): 
                line.set_color(self.axes_colour)
            
            for line in axes_top_histogram.xaxis.get_ticklines(): 
                line.set_color(self.axes_colour)

            for loc, spine in axes_top_histogram.spines.iteritems():
                    if loc in ['left','top']:
                            spine.set_color('none')
                    else:
                        spine.set_color(self.axes_colour)

            for a in axes_right_histogram.yaxis.majorTicks:
                    a.tick1On=True
                    a.tick2On=False
                
            for a in axes_right_histogram.xaxis.majorTicks:
                    a.tick1On=True
                    a.tick2On=False
                    
            for line in axes_right_histogram.yaxis.get_ticklines(): 
                line.set_color(self.axes_colour)
            
            for line in axes_right_histogram.xaxis.get_ticklines(): 
                line.set_color(self.axes_colour)

            for loc, spine in axes_right_histogram.spines.iteritems():
                    if loc in ['right','top']:
                            spine.set_color('none') 
                    else:
                        spine.set_color(self.axes_colour)

        #self.fig.tight_layout(pad=1.0, w_pad=0.1, h_pad=0.1)
        self.draw()



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__ = 'Donovan Parks'
__copyright__ = 'Copyright 2014'
__credits__ = ['Donovan Parks']
__license__ = 'GPL3'
__maintainer__ = 'Donovan Parks'
__email__ = 'donovan.parks@gmail.com'

import sys
from collections import namedtuple

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib.transforms as mtransforms

from matplotlib.patches import Rectangle

import matplotlib as mpl

import numpy as np


class AbstractPlot(FigureCanvas):
    """Abstract base class for plotting."""
    
    Options = namedtuple('Options', 'width height label_font_size tick_font_size dpi')

    def __init__(self, options):
        """Initialization."""

        if options:
            self.options = options
        else:
            Options = namedtuple('Options', 'width height label_font_size tick_font_size dpi')
            self.options = Options(6, 6, 10, 8, 300)

        self.set_font_size(self.options.label_font_size, self.options.tick_font_size)
        self.fig = Figure(facecolor='white', dpi=self.options.dpi)

        FigureCanvas.__init__(self, self.fig)

        self.cid = None

        self.type = '<none>'
        self.name = '<none>'

        self.axes_colour = (0.5, 0.5, 0.5)

    def set_font_size(self, label_size, tick_size):
        """Set font size for all text elements."""
        mpl.rcParams['font.size'] = label_size
        mpl.rcParams['axes.titlesize'] = label_size
        mpl.rcParams['axes.labelsize'] = label_size
        mpl.rcParams['xtick.labelsize'] = tick_size
        mpl.rcParams['ytick.labelsize'] = tick_size
        mpl.rcParams['legend.fontsize'] = label_size
        mpl.rcParams['svg.fonttype'] = 'none'

    def save_plot(self, filename, dpi=300):
        """Save plot to file."""

        imgFormat = filename[filename.rfind('.') + 1:len(filename)]
        if imgFormat in ['png', 'pdf', 'ps', 'eps', 'svg']:
            self.fig.savefig(filename, format=imgFormat, dpi=dpi, facecolor='white', edgecolor='white', bbox_inches='tight')
        else:
            pass

    def save_html(self, output_html, html_script=None, html_body=None):
        """Save figure as HTML.

        Parameters
        ----------
        output_html : str
            Name of output file.
        html_script : str
            Additional java script to append to script section.
        html_body : str
            Additional HTML to append to end of the body section.
        """

        try:
            __import__('mpld3')
        except ImportError:
            print '[Error] The mpld3 module is required to save HTML figures.'
            sys.exit()

        import mpld3

        # modify figure properties for better web viewing
        self.fig.dpi = 96
        html_str = mpld3.fig_to_html(self.fig, template_type='simple')

        if html_script:
            html_script_start = html_str.find('<script type="text/javascript">') + len('<script type="text/javascript">')
            html_str = html_str[0:html_script_start] + '\n' + html_script + '\n' + html_str[html_script_start:]

        if html_body:
            html_str += '\n<body>\n' + html_body + '\n</body>\n'

        html_str = '<center>' + html_str + '</center>'

        fout = open(output_html, 'w')
        fout.write(html_str)
        fout.close()

        # restore figure properties
        self.fig.dpi = self.options.dpi

    def prettify(self, axis):
        """Modify axis properties to make a cleaner plot."""

        for a in axis.yaxis.majorTicks:
            a.tick1On = True
            a.tick2On = False

        for a in axis.xaxis.majorTicks:
            a.tick1On = True
            a.tick2On = False

        for line in axis.yaxis.get_ticklines():
            line.set_color(self.axes_colour)

        for line in axis.xaxis.get_ticklines():
            line.set_color(self.axes_colour)

        for loc, spine in axis.spines.iteritems():
            if loc in ['right', 'top']:
                spine.set_color('none')
            else:
                spine.set_color(self.axes_colour)

    def label_extents(self, xLabels, xFontSize, xRotation, yLabels, yFontSize, yRotation):
        """Get maximum extents of labels."""

        self.fig.clear()

        tempAxes = self.fig.add_axes([0, 0, 1.0, 1.0])

        tempAxes.set_xticks(np.arange(len(xLabels)))
        tempAxes.set_yticks(np.arange(len(yLabels)))

        xText = tempAxes.set_xticklabels(xLabels, size=xFontSize, rotation=xRotation)
        yText = tempAxes.set_yticklabels(yLabels, size=yFontSize, rotation=yRotation)

        bboxes = []
        for label in xText:
            bbox = label.get_window_extent(self.get_renderer())
            bboxi = bbox.inverse_transformed(self.fig.transFigure)
            bboxes.append(bboxi)
        xLabelBounds = mtransforms.Bbox.union(bboxes)

        bboxes = []
        for label in yText:
            bbox = label.get_window_extent(self.get_renderer())
            bboxi = bbox.inverse_transformed(self.fig.transFigure)
            bboxes.append(bboxi)
        yLabelBounds = mtransforms.Bbox.union(bboxes)

        self.fig.clear()

        return xLabelBounds, yLabelBounds

    def x_label_extents(self, labels, fontSize, rotation=0):
        """Get maximum extents of x-axis labels."""

        self.fig.clear()

        tempAxes = self.fig.add_axes([0, 0, 1.0, 1.0])
        tempAxes.set_xticks(np.arange(len(labels)))
        xLabels = tempAxes.set_xticklabels(labels, size=fontSize, rotation=rotation)

        bboxes = []
        for label in xLabels:
            bbox = label.get_window_extent(self.get_renderer())
            bboxi = bbox.inverse_transformed(self.fig.transFigure)
            bboxes.append(bboxi)
        xLabelBounds = mtransforms.Bbox.union(bboxes)

        self.fig.clear()

        return xLabelBounds

    def y_label_extents(self, labels, fontSize, rotation=0):
        """Get maximum extents of y-axis labels."""

        self.fig.clear()

        tempAxes = self.fig.add_axes([0, 0, 1.0, 1.0])
        tempAxes.set_yticks(np.arange(len(labels)))
        yLabels = tempAxes.set_yticklabels(labels, size=fontSize, rotation=rotation)

        bboxes = []
        for label in yLabels:
            bbox = label.get_window_extent(self.get_renderer())
            bboxi = bbox.inverse_transformed(self.fig.transFigure)
            bboxes.append(bboxi)
        yLabelBounds = mtransforms.Bbox.union(bboxes)

        self.fig.clear()

        return yLabelBounds

    def format_labels(self, labels):
        """Make labels friendly to humans."""

        formattedLabels = []
        for label in labels:
            value = float(label.get_text())
            if value < 0.01:
                valueStr = '%.2e' % value
                if 'e-00' in valueStr:
                    valueStr = valueStr.replace('e-00', 'e-')
                elif 'e-0' in valueStr:
                    valueStr = valueStr.replace('e-0', 'e-')
            else:
                valueStr = '%.3f' % value

            formattedLabels.append(valueStr)

        return formattedLabels

    def remove_extra_zeros(self, label):
        """Remove unncessary trailing zeros from labels."""

        if '.' in label:
            while label[-1] == '0':
                label = label[0:-1]

        if label[-1] == '.':  # remove potential trailing decimal point
            label = label[0:-1]

        return label

    def bounding_box(self, data, ax, label, bBoundingBoxes, bLabels):
        """Draw bounding box around data."""

        data = np.array(data)

        width = max(data[:, 0]) - min(data[:, 0])
        height = max(data[:, 1]) - min(data[:, 1])
        r = Rectangle((min(data[:, 0]), min(data[:, 1])), width, height)

        if bBoundingBoxes:
            ax.add_artist(r)
            r.set_clip_box(ax.bbox)
            r.set_alpha(0.1)
            r.set_facecolor((0.5, 0.5, 0.5))

        if bLabels:
            ax.annotate(label, xy=(min(data[:, 0]), max(data[:, 1])), xytext=(0, 0),
                            textcoords='offset points', ha='right', va='bottom',
                            bbox=dict(boxstyle='round,pad=0.5', fc=(0.5, 0.5, 0.5), alpha=0.1))



###############################################################################
#                                                                             #
#    This program is free software: you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation, either version 3 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program. If not, see <http://www.gnu.org/licenses/>.     #
#                                                                             #
###############################################################################

__author__ = "Donovan Parks"
__copyright__ = "Copyright 2015"
__credits__ = ["Donovan Parks"]
__license__ = "GPL3"
__maintainer__ = "Donovan Parks"
__email__ = "donovan.parks@gmail.com"
__status__ = "Development"

import os
import logging

from biolib.external.execute import check_on_path


class Muscle():
    """Wrapper for running muscle."""

    def __init__(self):
        """Initialization."""
        self.logger = logging.getLogger('timestamp')

        check_on_path('muscle')

    def run(self, seqs, output_file, log_file):
        """Apply muscle to query sequences.

        Parameters
        ----------
        seqs : str
            Fasta file with sequence to alignment.
        output_file: str
            Output file containing multiple sequence alignment.
        log_file: str
            Output file containing information about running of muscle.
        """

        cmd = 'muscle -quiet -in %s -out %s -log %s' % (seqs, output_file, log_file)
        os.system(cmd)



from distutils.core import setup

import os


def version():
    setupDir = os.path.dirname(os.path.realpath(__file__))
    versionFile = open(os.path.join(setupDir, 'biolib', 'VERSION'))
    return versionFile.read().strip()

setup(
    name='biolib',
    version=version(),
    author='Donovan Parks',
    author_email='donovan.parks@gmail.com',
    packages=['biolib', 'biolib.misc', 'biolib.external', 'biolib.plots'],
    package_data={'biolib': ['VERSION']},
    url='http://pypi.python.org/pypi/biolib/',
    license='GPL3',
    description='Package for common tasks in bioinformatic.',
    long_description=open('README.md').read(),
    install_requires=[],
)
